@@ -46,6 +46,16 @@ module Net; module SSH; module Connection
     # The list of callbacks for pending requests. See #send_global_request.
     attr_reader :pending_requests #:nodoc:
 
+    class NilChannel
+      def initialize(session)
+        @session = session
+      end
+
+      def method_missing(sym, *args)
+        @session.lwarn { "ignoring request #{sym.inspect} for non-existent (closed?) channel; probably ssh server bug" }
+      end
+    end
+
     # Create a new connection service instance atop the given transport
     # layer. Initializes the listeners to be only the underlying socket object.
     def initialize(transport, options={})
@@ -55,7 +65,7 @@ module Net; module SSH; module Connection
       @options = options
 
       @channel_id_counter = -1
-      @channels = {}
+      @channels = Hash.new(NilChannel.new(self))
       @listeners = { transport.socket => nil }
       @pending_requests = []
       @channel_open_handlers = {}
@@ -100,6 +110,14 @@ module Net; module SSH; module Connection
       transport.close
     end
 
+    # Performs a "hard" shutdown of the connection. In general, this should
+    # never be done, but it might be necessary (in a rescue clause, for instance,
+    # when the connection needs to close but you don't know the status of the
+    # underlying protocol's state).
+    def shutdown!
+      transport.shutdown!
+    end
+
     # preserve a reference to Kernel#loop
     alias :loop_forever :loop
 
@@ -575,4 +593,4 @@ module Net; module SSH; module Connection
       end
   end
 
-end; end; end
\ No newline at end of file
+end; end; end
